# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010 Nexedi SA and Contributors. All Rights Reserved.
# Copyright (c) 2006-2008 Zope Corporation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import os
from string import Template
import logging
import zc.buildout
import zc.buildout.easy_install
import zc.recipe.egg
import ConfigParser

class WithMinusTemplate(Template):
  idpattern = '[_a-z][-_a-z0-9]*'

class Recipe(object):
  def __init__(self, buildout, name, options):
    self.buildout, self.options, self.name = buildout, options, name
    self.egg = zc.recipe.egg.Egg(buildout, options['recipe'], options)
    self.logger=logging.getLogger(self.name)


  def install_script(self, script_name):
    """ Create default scripts
    """
    self.logger.info('Installing memcached wrapper script...')
    options = self.options
    memcached_software_bin = options.get("memcached_software_bin")
    memcached_conf_file = options.get("memcached_conf_file")
    # Warning : if python2.4 is not installed with buildout you will get a
    # "ValueError: too many values to unpack" error.
    # To avoid this, make sure python2.4 is installed by buildout
    # (bin/buildout install python2.4)
    # See https://bugs.launchpad.net/zc.buildout/+bug/421833 -
    requirements, ws = self.egg.working_set(['erp5.recipe.memcachedserver'])
    zc.buildout.easy_install.scripts(
      [(script_name,'erp5.recipe.memcachedserver.ctl', 'main')],
      ws, options['executable'], options.get("memcached_bin_folder"),
      arguments = ("\n        '%s/%s' ,"
                   "\n        '%s' " % (memcached_software_bin,
                                         script_name,
                                         memcached_conf_file)))

  def build_configuration_file(self):
    config_file_location = self.options.get('memcached_conf_file')
    tcp_port = self.options.get('memcached_tcp_port')
    udp_port = self.options.get('memcached_udp_port')
    host = self.options.get('memcached_host')
    memory_size = self.options.get('memcached_memory_size')
    simultaneous_connections = self.options.get(
      'memcached_simultaneous_connections')
    user = self.options.get('memcached_user')
    run_as_daemon = self.options.get('memcached_run_as_daemon')
    pid_file = self.options.get('memcached_pid_file')
    factor = self.options.get('memcached_factor')
    key_size = self.options.get('memcached_key_size')
    threads = self.options.get('memcached_threads')
  
    # Checks configuration directory existence. If not, creates it
    confpath = os.path.dirname(config_file_location)
    if not os.path.isdir(confpath):
      os.makedirs(confpath)
    
    # Creates config file
    self.logger.info('Creating memcached config file...')
    config = ConfigParser.SafeConfigParser()
    config.add_section('memcached')
    if tcp_port:
      config.set('memcached', 'tcp_port', tcp_port)
    if udp_port:
       config.set('memcached', 'udp_port', udp_port)
    if host:
      config.set('memcached', 'host', host)
    if memory_size:
      config.set('memcached', 'memory_size', memory_size)
    if simultaneous_connections:
      config.set('memcached', 'simultaneous_connections', 
        simultaneous_connections)
    if user: 
      config.set('memcached', 'user', user)
    if run_as_daemon:
      config.set('memcached', 'run_as_daemon', run_as_daemon)
    if pid_file: 
      config.set('memcached', 'pid_file', pid_file)
    if factor: 
      config.set('memcached', 'factor', factor)
    if key_size: 
      config.set('memcached', 'key_size', key_size)
    if threads: 
      config.set('memcached', 'threads', threads)

    try:
      configfile = open(config_file_location, 'wb')
      config.write(configfile)
      configfile.close()
    except IOError:
      self.logger.error('Error writing memcached config file : %s',
        config_file_location)
      raise zc.buildout.UserError('Error writing memcached config file...')


  def install(self):
    self.build_configuration_file()
    self.install_script('memcached')
    return []

  update = install
